const db = getDb("cache.sqlite");
db.table(
  "blob_assets",
  /* SQL */ `
  create table blob_assets (
    hash text primary key,
    refs integer not null default 0
  );
`,
);
/**
 * Uncompressed files are read directly from the media store root.  Compressed
 * files are stored as `<compress store>/<first 2 chars of hash>/<hash>` Since
 * multiple files can share the same hash, the number of references is tracked
 * so that when a file is deleted, the compressed data is only removed when all
 * references are gone.
 */
export class BlobAsset {
  /** sha1 of the contents */
  hash!: string;
  refs!: number;
}

export namespace BlobAsset {
  const getQuery = cache.prepare(/* SQL */ `
    SELECT * FROM blob_assets WHERE hash = ?;
  `).as(BlobAsset);
  export function get(hash: string) {
    return getQuery.get(hash);
  }

  const putOrIncrementQuery = cache.prepare(/* SQL */ `
    INSERT INTO blob_assets (hash, refs) VALUES (?, 1)
    ON CONFLICT(hash) DO UPDATE SET refs = refs + 1;
  `);
  export function putOrIncrement(hash: string) {
    assert(hash.length === 40);
    putOrIncrementQuery.get(hash);
    return get(hash)!;
  }

  const decrementQuery = cache.prepare(/* SQL */ `
    UPDATE blob_assets SET refs = refs - 1 WHERE hash = ? AND refs > 0;
  `);
  const deleteQuery = cache.prepare(/* SQL */ `
    DELETE FROM blob_assets WHERE hash = ? AND refs <= 0;
  `);
  export function decrementOrDelete(hash: string) {
    assert(hash.length === 40);
    decrementQuery.run(hash);
    return deleteQuery.run(hash).changes > 0;
  }
}

import { getDb } from "#sqlite";
