// __esModule
true

// DiagnosticType
[object Object]

// assertAllowedAttributes
function assertAllowedAttributes(path, allowed) {
  let i = 0;
  for (const attr of path.node.attributes) {
    if (attr.type === "MarkoSpreadAttribute") {
      throw path.hub.buildError(
        attr,
        `Tag does not support spread attributes.`
      );
    } else if (!allowed.includes(attr.name)) {
      throw path.hub.buildError(
        attr,
        `Tag does not support the \`${attr.name}\` attribute.`
      );
    }

    i++;
  }
}

// assertAttributesOrArgs
function assertAttributesOrArgs(path) {
  const { node } = path;
  const args = node.arguments;
  if (args && args.length && (node.attributes.length > 0 || node.body.length)) {
    const start = args[0].loc.start;
    const end = args[args.length - 1].loc.end;
    throw path.hub.buildError(
      { loc: { start, end } },
      "Tag does not support arguments when attributes or body present."
    );
  }
}

// assertAttributesOrSingleArg
function assertAttributesOrSingleArg(path) {
  assertAttributesOrArgs(path);
  const args = path.node.arguments;
  if (args && args.length > 1) {
    const start = args[1].loc.start;
    const end = args[args.length - 1].loc.end;
    throw path.hub.buildError(
      { loc: { start, end } },
      "Tag does not support multiple arguments."
    );
  }
}

// assertNoArgs
function assertNoArgs(path) {
  const args = path.node.arguments;
  if (args && args.length) {
    const start = args[0].loc.start;
    const end = args[args.length - 1].loc.end;
    throw path.hub.buildError(
      { loc: { start, end } },
      "Tag does not support arguments."
    );
  }
}

// assertNoAttributeTags
function assertNoAttributeTags(path) {
  if (path.node.attributeTags.length) {
    throw path.hub.buildError(
      path.node.attributeTags[0],
      "Tag not support nested attribute tags."
    );
  }
}

// assertNoAttributes
function assertNoAttributes(path) {
  const { attributes } = path.node;
  if (attributes.length) {
    const start = attributes[0].loc.start;
    const end = attributes[attributes.length - 1].loc.end;
    throw path.hub.buildError(
      { loc: { start, end } },
      "Tag does not support attributes."
    );
  }
}

// assertNoParams
function assertNoParams(path) {
  const { params } = path.node.body;
  if (params.length) {
    const start = params[0].loc.start;
    const end = params[params.length - 1].loc.end;
    throw path.hub.buildError(
      { loc: { start, end } },
      "Tag does not support parameters."
    );
  }
}

// assertNoVar
function assertNoVar(path) {
  if (path.node.var) {
    throw path.hub.buildError(
      path.node.var,
      "Tag does not support a variable."
    );
  }
}

// computeNode
function computeNode(node) {
  switch (node.type) {
    case "StringLiteral":
    case "NumericLiteral":
    case "BooleanLiteral":
      return { value: node.value };
    case "RegExpLiteral":
      return { value: new RegExp(node.pattern, node.flags) };
    case "NullLiteral":
      return { value: null };
    case "Identifier":
      switch (node.name) {
        case "undefined":
          return { value: undefined };
        case "NaN":
          return { value: NaN };
        case "Infinity":
          return { value: Infinity };
        default:
          return;
      }
    case "BigIntLiteral":
      return { value: BigInt(node.value) };
    case "ParenthesizedExpression":
      return computeNode(node.expression);
    case "BinaryExpression":{
        const left = computeNode(node.left);
        if (!left) return;
        const right = computeNode(node.right);
        if (!right) return;
        switch (node.operator) {
          case "+":
            return { value: left.value + right.value };
          case "-":
            return { value: left.value - right.value };
          case "*":
            return { value: left.value * right.value };
          case "/":
            return { value: left.value / right.value };
          case "%":
            return { value: left.value % right.value };
          case "**":
            return { value: left.value ** right.value };
          case "|":
            return { value: left.value | right.value };
          case "&":
            return { value: left.value & right.value };
          case "^":
            return { value: left.value ^ right.value };
          case "<<":
            return { value: left.value << right.value };
          case ">>":
            return { value: left.value >> right.value };
          case ">>>":
            return { value: left.value >>> right.value };
          case "==":
            return { value: left.value == right.value };
          case "!=":
            return { value: left.value != right.value };
          case "===":
            return { value: left.value === right.value };
          case "!==":
            return { value: left.value !== right.value };
          case "<":
            return { value: left.value < right.value };
          case "<=":
            return { value: left.value <= right.value };
          case ">":
            return { value: left.value > right.value };
          case ">=":
            return { value: left.value >= right.value };
          default:
            return;
        }
      }
    case "UnaryExpression":{
        const arg = computeNode(node.argument);
        if (!arg) return;
        switch (node.operator) {
          case "+":
            return { value: +arg.value };
          case "-":
            return { value: -arg.value };
          case "~":
            return { value: ~arg.value };
          case "!":
            return { value: !arg.value };
          case "typeof":
            return { value: typeof arg.value };
          case "void":
            return { value: void arg.value };
          default:
            return;
        }
      }
    case "LogicalExpression":{
        const left = computeNode(node.left);
        if (!left) return;
        const right = computeNode(node.right);
        if (!right) return;
        switch (node.operator) {
          case "&&":
            return { value: left.value && right.value };
          case "||":
            return { value: left.value || right.value };
          case "??":
            return { value: left.value ?? right.value };
          default:
            return;
        }
      }
    case "ConditionalExpression":{
        const test = computeNode(node.test);
        if (!test) return;
        const consequent = computeNode(node.consequent);
        if (!consequent) return;
        const alternate = computeNode(node.alternate);
        if (!alternate) return;
        return { value: test.value ? consequent.value : alternate.value };
      }
    case "TemplateLiteral":{
        let value = node.quasis[0].value.cooked;
        for (let i = 0; i < node.expressions.length; i++) {
          const expr = computeNode(node.expressions[i]);
          if (!expr) return;
          value += expr.value + node.quasis[i + 1].value.cooked;
        }
        return { value };
      }
    case "ObjectExpression":{
        const value = {};
        for (const prop of node.properties) {
          if (prop.decorators) return;
          switch (prop.type) {
            case "ObjectProperty":{
                let key;
                if (prop.computed) {
                  const keyNode = computeNode(prop.key);
                  if (!keyNode) return;
                  key = keyNode.value + "";
                } else {
                  switch (prop.key.type) {
                    case "Identifier":
                      key = prop.key.name;
                      break;
                    case "StringLiteral":
                      key = prop.key.value;
                      break;
                    default:
                      return;
                  }
                }

                const propValue = computeNode(prop.value);
                if (!propValue) return;
                value[key] = propValue.value;
                break;
              }
            case "SpreadElement":{
                const arg = computeNode(prop.argument);
                if (!arg) return;
                Object.assign(value, arg.value);
                break;
              }
          }
        }

        return { value };
      }
    case "ArrayExpression":{
        const value = [];
        for (const elem of node.elements) {
          if (elem) {
            if (elem.type === "SpreadElement") {
              const arg = computeNode(elem.argument);
              if (typeof arg?.value?.[Symbol.iterator] !== "function") return;
              for (const item of arg.value) {
                value.push(item);
              }
            } else {
              const elemValue = computeNode(elem);
              if (!elemValue) return;
              value.push(elemValue.value);
            }
          } else {
            value.length++;
          }
        }

        return { value };
      }
  }
}

// defineTag
function defineTag(tag) {
  return tag;
}

// diagnosticDeprecate
function diagnosticDeprecate(path, options) {
  add(DiagnosticType.Deprecation, path, options);
}

// diagnosticError
function diagnosticError(path, options) {
  add(DiagnosticType.Error, path, options);
}

// diagnosticSuggest
function diagnosticSuggest(path, options) {
  add(DiagnosticType.Suggestion, path, options);
}

// diagnosticWarn
function diagnosticWarn(path, options) {
  add(DiagnosticType.Warning, path, options);
}

// findAttributeTags
function findAttributeTags(path, attributeTags = []) {
  const attrTags = path.node.body.attributeTags ?
  path.get("body").get("body") :
  path.get("attributeTags");
  attrTags.forEach((child) => {
    if (isAttributeTag(child)) {
      attributeTags.push(child);
    } else if (isTransparentTag(child)) {
      findAttributeTags(child, attributeTags);
    }
  });

  return attributeTags;
}

// findParentTag
function findParentTag(path) {
  let cur = path.parentPath;

  while (cur.node) {
    if (cur.isMarkoTagBody()) {
      cur = cur.parentPath;
      continue;
    }

    if (!cur.isMarkoTag()) {
      cur = undefined;
      break;
    }

    if (isTransparentTag(cur)) {
      cur = cur.parentPath;
      continue;
    }

    return cur;
  }
}

// getArgOrSequence
function getArgOrSequence(path) {
  const {
    node: { arguments: args }
  } = path;
  const len = args && args.length;

  if (len) {
    if (len > 1) {
      return _compiler.types.sequenceExpression(args);
    } else {
      return args[0];
    }
  }
}

// getEnd
function getEnd(file, node) {
  // Restore if merged: https://github.com/babel/babel/pull/16849
  // if (node.end != null) {
  //   return node.end;
  // }

  if (node.loc) {
    return locToIndex(file, node.loc.end);
  }

  return null;
}

// getFile
function getFile() {
  if (currentFile) {
    return currentFile;
  }

  throw new Error("Unable to access Marko File outside of a compilation");
}

// getFullyResolvedTagName
function getFullyResolvedTagName(path) {
  const parts = [];
  let cur;
  do {
    cur = path.node.name.value;

    if (isAttributeTag(path)) {
      parts.push(cur.slice(1));
    } else {
      parts.push(cur || "*");
      break;
    }
  } while (path = findParentTag(path));

  return parts.reverse().join(":");
}

// getLoc
function getLoc(file, index) {
  return findLoc(getLineIndexes(file), 0, index);
}

// getLocRange
function getLocRange(file, start, end) {
  const lineIndexes = getLineIndexes(file);
  const startLoc = findLoc(lineIndexes, 0, start);

  if (startLoc) {
    const endLoc =
    start === end ? startLoc : findLoc(lineIndexes, startLoc.line - 1, end);

    return {
      start: startLoc,
      end: endLoc
    };
  }
}

// getMacroIdentifier
function getMacroIdentifier(path) {
  const { file } = path.hub;

  if (file.___compileStage !== "translate") {
    throw new Error(
      "getMacroIdentifier can only be called during the translate phase of the compiler."
    );
  }

  if (!isMacroTag(path)) {
    throw path.buildCodeFrameError(
      "getMacroIdentifier called on non macro referencing tag."
    );
  }

  return getMacroIdentifierForName(path, path.node.name.value);
}

// getMacroIdentifierForName
function getMacroIdentifierForName(path, name) {
  const { file } = path.hub;

  if (file.___compileStage !== "translate") {
    throw new Error(
      "getMacroIdentifierForName can only be called during the translate phase of the compiler."
    );
  }

  const markoMeta = file.metadata.marko;
  let macroIds = markoMeta[MACRO_IDS_KEY];

  if (!macroIds) {
    macroIds = markoMeta[MACRO_IDS_KEY] = {};

    for (const macroName in markoMeta[MACRO_NAMES_KEY]) {
      macroIds[macroName] = file.path.scope.generateUid(macroName);
    }
  }

  const id = macroIds[name];

  if (!id) {
    throw new Error(
      "<macro> was added programmatically, but was not registered via the 'registerMacro' api in @marko/compiler/babel-utils."
    );
  }

  return _compiler.types.identifier(id);
}

// getProgram
function getProgram() {
  if (currentFile) {
    return currentFile.path;
  }

  throw new Error("Unable to access Marko Program outside of a compilation");
}

// getStart
function getStart(file, node) {
  // Restore if merged: https://github.com/babel/babel/pull/16849
  // if (node.start != null) {
  //   return node.start;
  // }

  if (node.loc) {
    return locToIndex(file, node.loc.start);
  }

  return null;
}

// getTagDef
function getTagDef(path) {
  const {
    node,
    hub: { file }
  } = path;

  if (node.tagDef === undefined) {
    if (isDynamicTag(path) || isMacroTag(path)) {
      node.tagDef = null;
    } else {
      node.tagDef =
      (0, _taglib.getTagDefForTagName)(
        file,
        isAttributeTag(path) ?
        getFullyResolvedTagName(path) :
        node.name.value
      ) || null;
    }
  }

  return node.tagDef;
}

// getTagDefForTagName
function getTagDefForTagName(file, tagName) {
  const tagDef = getTaglibLookup(file).getTag(tagName);

  if (tagDef) {
    let seen = file.metadata.marko[SEEN_TAGS_KEY];
    if (!seen) {
      seen = file.metadata.marko[SEEN_TAGS_KEY] = new Set();
    }

    if (!seen.has(tagDef)) {
      seen.add(tagName);
      const { filePath } = tagDef;
      const len = filePath.length;

      if (filePath[len - 14] === "m" && filePath.endsWith("marko-tag.json")) {
        file.metadata.marko.watchFiles.push(filePath);
      }
    }
  }
  return tagDef;
}

// getTagTemplate
function getTagTemplate(tag) {
  const {
    node,
    hub: { file }
  } = tag;

  if (node.extra?.tagNameImported) {
    return (0, _path.join)(file.opts.filename, node.extra.tagNameImported);
  }
  return getTagDef(tag)?.template;
}

// getTaglibLookup
function getTaglibLookup(file) {
  return file.___taglibLookup;
}

// getTemplateId
function getTemplateId(opts, request, child) {
  const id = (0, _path.relative)(ROOT, request).replace(/[^a-zA-Z0-9_$./-]/g, "/");
  const optimize = typeof opts === "object" ? opts.optimize : opts;

  if (optimize) {
    const optimizeKnownTemplates =
    typeof opts === "object" && opts.optimizeKnownTemplates;
    const knownTemplatesSize = optimizeKnownTemplates?.length || 0;
    if (knownTemplatesSize) {
      let lookup = idCache.get(optimizeKnownTemplates);
      if (!lookup) {
        lookup = new Map();
        idCache.set(optimizeKnownTemplates, lookup);
        for (let i = 0; i < knownTemplatesSize; i++) {
          lookup.set(optimizeKnownTemplates[i], {
            id: encodeTemplateId(i),
            children: new Map()
          });
        }
      }
      let registered = lookup.get(request);
      if (registered) {
        if (child) {
          let childId = registered.children.get(child);
          if (childId === undefined) {
            childId = registered.children.size;
            registered.children.set(child, childId);
          }
          return registered.id + childId;
        }
        return registered.id;
      }
    }

    const hash = (0, _crypto.createHash)("shake256", templateIdHashOpts).update(id);
    if (child) {
      hash.update(child);
    }

    return encodeTemplateId(parseInt(hash.digest("hex"), 16));
  }

  return id + (child ? `_${child}` : "");
}

// hasMacro
function hasMacro(path, name) {
  const macroNames = path.hub.file.metadata.marko[MACRO_NAMES_KEY];
  return !!(macroNames && macroNames[name]);
}

// importDefault
function importDefault(file, request, nameHint) {
  const imports = getImports(file);
  request = resolveRelativePath(file, request);
  let importDeclaration = imports.get(request);

  if (!importDeclaration) {
    imports.set(
      request,
      importDeclaration = file.path.pushContainer(
        "body",
        _compiler.types.importDeclaration([], _compiler.types.stringLiteral(request))
      )[0]
    );
  }

  if (!nameHint) {
    return;
  }

  const specifiers = importDeclaration.get("specifiers");
  const specifier = specifiers.find((specifier) =>
  specifier.isImportDefaultSpecifier()
  );

  if (!specifier) {
    const identifier = file.scope.generateUidIdentifier(nameHint);
    importDeclaration.pushContainer(
      "specifiers",
      _compiler.types.importDefaultSpecifier(identifier)
    );
    return identifier;
  }

  return _compiler.types.identifier(specifier.node.local.name);
}

// importNamed
function importNamed(file, request, name, nameHint = name) {
  request = resolveRelativePath(file, request);
  const imports = getImports(file);
  let importDeclaration = imports.get(request);

  if (!importDeclaration) {
    imports.set(
      request,
      importDeclaration = file.path.pushContainer(
        "body",
        _compiler.types.importDeclaration([], _compiler.types.stringLiteral(request))
      )[0]
    );
  }

  const specifiers = importDeclaration.get("specifiers");
  const specifier = specifiers.find(
    (specifier) =>
    specifier.isImportSpecifier() && specifier.node.imported.name === name
  );

  if (!specifier) {
    const identifier = file.scope.generateUidIdentifier(nameHint);
    importDeclaration.pushContainer(
      "specifiers",
      _compiler.types.importSpecifier(identifier, _compiler.types.identifier(name))
    );
    return identifier;
  }

  return _compiler.types.identifier(specifier.node.local.name);
}

// importStar
function importStar(file, request, nameHint) {
  const imports = getImports(file);
  request = resolveRelativePath(file, request);
  let importDeclaration = imports.get(request);

  if (!importDeclaration) {
    imports.set(
      request,
      importDeclaration = file.path.pushContainer(
        "body",
        _compiler.types.importDeclaration([], _compiler.types.stringLiteral(request))
      )[0]
    );
  }

  if (!nameHint) {
    return;
  }

  const specifiers = importDeclaration.get("specifiers");
  const specifier = specifiers.find((specifier) =>
  specifier.isImportNamespaceSpecifier()
  );

  if (!specifier) {
    const identifier = file.scope.generateUidIdentifier(nameHint);
    importDeclaration.pushContainer(
      "specifiers",
      _compiler.types.importNamespaceSpecifier(identifier)
    );
    return identifier;
  }

  return _compiler.types.identifier(specifier.node.local.name);
}

// isAttributeTag
function isAttributeTag(path) {
  const {
    node: { name }
  } = path;
  return _compiler.types.isStringLiteral(name) && name.value[0] === "@";
}

// isDynamicTag
function isDynamicTag(path) {
  return !_compiler.types.isStringLiteral(path.node.name);
}

// isLoopTag
function isLoopTag(path) {
  if (!path.isMarkoTag()) {
    return false;
  }

  const tagName = path.node.name.value;
  return tagName === "while" || tagName === "for";
}

// isMacroTag
function isMacroTag(path) {
  const { name } = path.node;
  return _compiler.types.isStringLiteral(name) && hasMacro(path, name.value);
}

// isNativeTag
function isNativeTag(path) {
  if (path.node._isDynamicString) {
    return true;
  }

  const tagDef = getTagDef(path);
  return (
    tagDef &&
    tagDef.html && (
    tagDef.htmlType === "custom-element" ||
    !tagDef.template && !tagDef.renderer));

}

// isTransparentTag
function isTransparentTag(path) {
  const {
    node: { name }
  } = path;
  return _compiler.types.isStringLiteral(name) && TRANSPARENT_TAGS.has(name.value);
}

// loadFileForImport
function loadFileForImport(file, request) {
  const relativeRequest = resolveTagImport(file.path, request);

  if (relativeRequest) {
    const filename =
    relativeRequest[0] === "." ?
    (0, _path.resolve)(file.opts.filename, "..", relativeRequest) :
    (0, _resolveFrom.default)((0, _path.dirname)(file.opts.filename), relativeRequest);
    const markoMeta = file.metadata.marko;
    const { analyzedTags } = markoMeta;
    if (analyzedTags) {
      analyzedTags.add(relativeRequest);
    } else {
      markoMeta.analyzedTags = new Set([relativeRequest]);
    }

    return resolveMarkoFile(file, filename);
  }
}

// loadFileForTag
function loadFileForTag(tag) {
  const { file } = tag.hub;
  if (tag.node.extra?.tagNameImported) {
    return loadFileForImport(file, tag.node.extra?.tagNameImported);
  }

  const def = getTagDef(tag);
  const filename = def && def.template;

  if (filename) {
    const markoMeta = file.metadata.marko;
    const relativeFileName = (0, _imports.resolveRelativePath)(file, filename);
    const { analyzedTags } = markoMeta;
    if (analyzedTags) {
      analyzedTags.add(relativeFileName);
    } else {
      markoMeta.analyzedTags = new Set([relativeFileName]);
    }

    return resolveMarkoFile(file, filename);
  }
}

// normalizeTemplateString
function normalizeTemplateString(quasis, ...expressions) {
  quasis = quasis.map((q) => _compiler.types.isTemplateElement(q) ? q.value.cooked : q);

  for (let i = expressions.length; i--;) {
    let v = expressions[i];
    if (_compiler.types.isTemplateLiteral(v)) {
      quasis[i] += v.quasis[0].value.cooked;
      quasis[i + 1] =
      v.quasis[v.quasis.length - 1].value.cooked + (quasis[i + 1] || "");
      quasis.splice(
        i + 1,
        0,
        ...v.quasis.slice(1, -1).map((q) => q.value.cooked)
      );
      expressions.splice(i, 1, ...v.expressions);
      i += v.expressions.length;
    } else if (_compiler.types.isStringLiteral(v) || typeof v === "string") {
      const value = _compiler.types.isStringLiteral(v) ? v.value : v;
      quasis[i] += value + quasis[i + 1];
      expressions.splice(i, 1);
      quasis.splice(i + 1, 1);
    }
  }

  if (!expressions.length) {
    // No expression, just return a literal or empty.
    const literal = quasis.join("");
    return literal === "" ? undefined : _compiler.types.stringLiteral(literal);
  }

  if (
  expressions.length === 1 &&
  quasis.length === 2 &&
  quasis.every(isEmptyString))
  {
    // Only expression `${expr}` just return the expr.
    return expressions[0];
  }

  // Do it.
  return _compiler.types.templateLiteral(quasis.map(getTemplateElement), expressions);
}

// parseArgs
function parseArgs(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(file, `_(${str})`, sourceStart, sourceEnd, 2);

  if (parsed.type === "CallExpression") {
    return parsed.arguments;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

// parseExpression
function parseExpression(
file,
str,
sourceStart,
sourceEnd,
sourceOffset)
{
  return tryParse(file, true, str, sourceStart, sourceEnd, sourceOffset);
}

// parseParams
function parseParams(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
    file,
    `(${str})=>{}`,
    sourceStart,
    sourceEnd,
    1
  );

  if (parsed.type === "ArrowFunctionExpression") {
    return parsed.params;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

// parseStatements
function parseStatements(
file,
str,
sourceStart,
sourceEnd,
sourceOffset)
{
  return tryParse(file, false, str, sourceStart, sourceEnd, sourceOffset);
}

// parseTemplateLiteral
function parseTemplateLiteral(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
    file,
    "`" + str + "`",
    sourceStart,
    sourceEnd,
    1
  );

  if (parsed.type === "TemplateLiteral") {
    return _compiler.types.templateLiteral(parsed.quasis, parsed.expressions);
  }

  return ensureParseError(file, parsed, sourceStart, sourceEnd);
}

// parseTypeArgs
function parseTypeArgs(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(file, `_<${str}>`, sourceStart, sourceEnd, 2);

  if (parsed.type === "TSInstantiationExpression") {
    // typeArguments is Flow only (not TS), we need to use typeParameters
    return parsed.typeParameters;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

// parseTypeParams
function parseTypeParams(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
    file,
    `<${str}>()=>{}`,
    sourceStart,
    sourceEnd,
    1
  );

  if (parsed.type === "ArrowFunctionExpression") {
    return parsed.typeParameters;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

// parseVar
function parseVar(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
    file,
    `(${str})=>{}`,
    sourceStart,
    sourceEnd,
    1
  );

  if (parsed.type === "ArrowFunctionExpression" && parsed.params.length === 1) {
    return parsed.params[0];
  }

  return ensureParseError(file, parsed, sourceStart, sourceEnd);
}

// registerMacro
function registerMacro(path, name) {
  const { file } = path.hub;
  const markoMeta = file.metadata.marko;
  const macroNames = markoMeta[MACRO_NAMES_KEY];

  if (macroNames) {
    if (macroNames[name]) {
      (0, _diagnostics.diagnosticWarn)(path, {
        label: `A macro with the name "${name}" already exists.`,
        fix() {
          findParentTag(path).remove();
        }
      });
    }
    macroNames[name] = true;
  } else {
    markoMeta[MACRO_NAMES_KEY] = { [name]: true };
  }
}

// resolveRelativePath
function resolveRelativePath(file, request) {
  if (request.startsWith(FS_START)) {
    request = (0, _relativeImportPath.relativeImportPath)(file.opts.filename, request);
  }

  if (file.markoOpts.optimize) {
    request = request.replace(
      /(^|\/node-modules\/)marko\/src\//,
      "$1marko/dist/"
    );
  }

  return request;
}

// resolveTagImport
function resolveTagImport(path, request) {
  const {
    hub: { file }
  } = path;
  if (request[0] === "<") {
    const tagName = request.slice(1, -1);
    const tagDef = (0, _taglib.getTagDefForTagName)(file, tagName);
    const tagEntry = tagDef && (tagDef.renderer || tagDef.template);
    const relativePath = tagEntry && (0, _imports.resolveRelativePath)(file, tagEntry);

    if (!relativePath) {
      throw path.buildCodeFrameError(
        `Unable to find entry point for custom tag <${tagName}>.`
      );
    }

    return relativePath;
  }

  if (request.endsWith(".marko")) {
    return (0, _imports.resolveRelativePath)(file, request);
  }
}

// withLoc
function withLoc(file, node, start, end) {
  node.loc = getLocRange(file, start, end);
  node.start = start;
  node.end = end;
  return node;
}
